# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Country',
            fields=[
                ('is_deleted', models.BooleanField(default=False)),
                ('iso', models.CharField(max_length=2, serialize=False, primary_key=True)),
                ('name', models.CharField(max_length=128)),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='EnvironmentalTheme',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('is_deleted', models.BooleanField(default=False)),
                ('title', models.CharField(max_length=128)),
            ],
            options={
                'ordering': ('-pk',),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='GeographicalScope',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('is_deleted', models.BooleanField(default=False)),
                ('title', models.CharField(max_length=128)),
                ('require_country', models.BooleanField(default=False)),
            ],
            options={
                'ordering': ('-pk',),
            },
            bases=(models.Model,),
        ),
    ]



import inspect
import sys

from django.db.models import BooleanField
from django.db.models import CharField
from django.db.models import Model


class ReplicatedModel(Model):

    class Meta:
        abstract = True
        permissions = (
            ('config', 'Can change configuration'),
        )

    is_deleted = BooleanField(default=False)


class GeographicalScope(ReplicatedModel):

    title = CharField(max_length=128)
    require_country = BooleanField(default=False)

    class Meta:
        ordering = ('-pk',)

    def __unicode__(self):
        return self.title


class EnvironmentalTheme(ReplicatedModel):

    title = CharField(max_length=128)

    class Meta:
        ordering = ('-pk',)

    def __unicode__(self):
        return self.title


class Country(ReplicatedModel):

    iso = CharField(max_length=2, primary_key=True)
    name = CharField(max_length=128)

    def __unicode__(self):
        return self.name


def _is_replicated_model(cls):
    """Check wether a class is a model to be replicated.

    This is implementend by extracting all models that inherit from
    ReplicatedModel
    """
    return (inspect.isclass(cls) and
            issubclass(cls, ReplicatedModel) and
            cls != ReplicatedModel)


def get_replicated_models():
    """Return all replicated models in this module."""
    return inspect.getmembers(sys.modules[__name__], _is_replicated_model)



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'GeographicalScope'
        db.create_table(u'common_geographicalscope', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('is_deleted', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=128)),
            ('require_country', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal(u'common', ['GeographicalScope'])

        # Adding model 'EnvironmentalTheme'
        db.create_table(u'common_environmentaltheme', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('is_deleted', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=128)),
        ))
        db.send_create_signal(u'common', ['EnvironmentalTheme'])

        # Adding model 'Country'
        db.create_table(u'common_country', (
            ('is_deleted', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('iso', self.gf('django.db.models.fields.CharField')(max_length=2, primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=128)),
        ))
        db.send_create_signal(u'common', ['Country'])


    def backwards(self, orm):
        # Deleting model 'GeographicalScope'
        db.delete_table(u'common_geographicalscope')

        # Deleting model 'EnvironmentalTheme'
        db.delete_table(u'common_environmentaltheme')

        # Deleting model 'Country'
        db.delete_table(u'common_country')


    models = {
        u'common.country': {
            'Meta': {'object_name': 'Country'},
            'is_deleted': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'iso': ('django.db.models.fields.CharField', [], {'max_length': '2', 'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'})
        },
        u'common.environmentaltheme': {
            'Meta': {'ordering': "('-pk',)", 'object_name': 'EnvironmentalTheme'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_deleted': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '128'})
        },
        u'common.geographicalscope': {
            'Meta': {'ordering': "('-pk',)", 'object_name': 'GeographicalScope'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_deleted': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'require_country': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '128'})
        }
    }

    complete_apps = ['common']


import requests
import json

from django.conf import settings
from django.db import transaction
from django.core.management.base import BaseCommand
from flis_metadata.common.models import get_replicated_models

from requests.exceptions import ConnectionError


class RemoteException(Exception):
    pass


class MaxRequestsReached(Exception):
    pass


MAX_REQUESTS = 100


def get_model_instances(name, model):
    """For a given model get all instances using the remote API."""
    endpoint = (settings.METADATA_REMOTE_HOST +
                '/api/v1/{0}/?format=json'.format(name.lower()))

    objects = []
    for _ in xrange(MAX_REQUESTS):
        try:
            response = requests.get(endpoint)
        except ConnectionError:
            raise RemoteException(
                'Error reaching remote endpoint: {0}'.format(endpoint))

        if response.status_code != 200:
            raise RemoteException(
                'Bad response for endpoint: {0}'.format(endpoint))

        response_obj = json.loads(response.text)

        objects += response_obj['objects']
        if not response_obj['meta']['next']:
            break

        endpoint = settings.METADATA_REMOTE_HOST + response_obj['meta']['next']
    else:
        raise MaxRequestsReached()

    # Remove default resource_uri field
    for o in objects:
        o.pop('resource_uri')

    return objects


@transaction.atomic
def update_model_instances(name, model):
    """Update local model for model data from remote instances."""
    instances = get_model_instances(name, model)

    for instance in instances:
        obj = model(**instance)
        obj.save()


class Command(BaseCommand):
    help = 'Sync remote models on local database'

    def handle(self, *args, **options):
        for name, model in get_replicated_models():
            print 'Syncing {0}: ...'.format(name),
            update_model_instances(name, model)
            print 'done!'



import os
from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='eea.flis.metadata',
    version='0.5.0',
    packages=['flis_metadata',
              'flis_metadata.client',
              'flis_metadata.client.management',
              'flis_metadata.client.management.commands',
              'flis_metadata.common',
              'flis_metadata.common.migrations',
              'flis_metadata.common.south_migrations'],
    install_requires=['requests',
                      'django'],
    include_package_data=True,
    license='BSD License',
    description='EEA Metadata models replication support',
    long_description=README,
    author='Mihai Bivol',
    url='https://github.com/eea/flis.metadata',
    author_email='mm.bivol@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
